<!---
	csw-common.cfc

	This component file contains a set on widget functions that are common to
	many of the CS Widgets.
	
	Version: [2018-01-03 12:00 pm]
--->

<cfcomponent displayname="csw common" output="false" hint="Common Widgets Functions">
	
	<!--- --------------------------------------- --->
	<!---        Common Rendering Functions       --->
	<!--- --------------------------------------- --->

	<!----------------------------------------
		Render Title
	----------------------------------------->
	<cffunction name="renderTitle" access="public" output="Yes" returntype="void">
		<cfargument name="dataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">

		<cfscript>
			var data = arguments.dataStruct;
			var props = arguments.propsStruct;
			var bgColor = '';
		
			RenderTextCore(
				class = 'cswTitle',
				tag = props.title.tag,
				text = data.title,
				color = props.title.color,
				backgroundColor = props.title.bgColor,
				size = props.title.size,
				align = props.title.align,
				fontClasses = props.title.fontClasses,
				padding = props.title.padding,
				effect = props.title.effect,
				linkURL = data.linkURL,
				style = '' );
		</cfscript>
	</cffunction>

	<!----------------------------------------
		Render Subtitle
	----------------------------------------->
	<cffunction name="renderSubtitle" access="public" output="Yes" returntype="void">
		<cfargument name="dataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">

		<cfscript>
			var data = arguments.dataStruct;
			var props = arguments.propsStruct;
		
			if( props.subtitle.show AND data.subtitle neq '' )
			{
				RenderTextCore(
					class = 'cswSubtitle',
					tag = props.subtitle.tag,
					text = data.subtitle,
					color = props.subtitle.color,
					backgroundColor = props.subtitle.bgColor,
					size = props.subtitle.size,
					align = props.subtitle.align,
					fontClasses = props.subtitle.fontClasses,
					padding = props.subtitle.padding,
					effect = props.subtitle.effect,
					linkURL = '',
					style = '' );
			}
		</cfscript>

	</cffunction>

	<!----------------------------------------
		Render Description
	----------------------------------------->
	<cffunction name="renderDescription" access="public" output="Yes" returntype="void">
		<cfargument name="dataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">

		<cfscript>
			var data = arguments.dataStruct;
			var props = arguments.propsStruct;
		
			if( props.desc.show AND data.desc neq '' )
			{
				RenderTextCore(
					class = 'cswDesc',
					tag = props.desc.tag,
					text = data.desc,
					color = props.desc.color,
					backgroundColor = props.desc.bgColor,
					size = '',
					align = props.desc.align,
					fontClasses = props.desc.fontClasses,
					padding = props.desc.padding,
					effect = props.desc.effect,
					linkURL = '',
					style = '' );
			}
		</cfscript>
	
	</cffunction>
	
	<!----------------------------------------
		Render Separator Line
	----------------------------------------->
	<cffunction name="renderSeparatorLine" access="public" output="Yes" returntype="void">
		<cfargument name="dataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">

		<cfscript>
			var data = arguments.dataStruct;
			var props = arguments.propsStruct;

			if( props.line.show )
			{
				renderLineCore( class = '', 
								 color = props.line.color, 
								 backgroundColor = props.line.bgColor,
								 size = props.line.size,
								 align =  props.line.align,
								 padding = props.line.padding,
								 effect = props.line.effect,
								 style = props.line.style );
			}			
		</cfscript>
				 
	</cffunction>

	<!--- -------------------------------------- --->
	<!---        Core Rendering Functions        --->
	<!--- -------------------------------------- --->

	<!----------------------------------------
		Render Text Core
	----------------------------------------->
	<cffunction name="renderTextCore" access="public" output="Yes" returntype="void">
		<cfargument name="class" type="string" required="Yes" hint="a class value">
		<cfargument name="tag" type="string" required="Yes">
		<cfargument name="text" type="string" required="Yes">
		<cfargument name="color" type="string" required="Yes" hint="a style value">
		<cfargument name="backgroundColor" type="string" required="Yes" hint="a style value">
		<cfargument name="size" type="string" required="no" default="" hint="a class value">
		<cfargument name="align" type="string" required="no" default="" hint="a class value">
		<cfargument name="fontClasses" type="string" required="no" default="" hint="a class value">
		<cfargument name="padding" type="string" required="no" default="" hint="a style value">
		<cfargument name="effect" type="string" required="no" default="" hint="a class value">
		<cfargument name="linkURL" type="string" required="no" default="">
		<cfargument name="linkClass" type="string" required="no" default="">
		<cfargument name="iconClass" type="string" required="no" default="">
		<cfargument name="style" type="string" required="no" default="" hint="a style value">
		<cfargument name="onClick" type="string" required="no" default="" hint="a onClick value">
		<cfargument name="linkAttrStruct" type="struct" required="no" default="#StructNew()#" hint="additional attributes key/value pairs">

		<cfscript>
			var classes = arguments.class;
			var linkClasses = '';
			var styleAttr = arguments.style;
			var theText = arguments.text;
			var linkStyleAttr = '';
			var onClickAttr = '';
			var linkAttr = '';
			var attrKey = '';
			var theIcon = '';

			// Classes
			if( Len(arguments.size) )
				classes = classes & ' ' & arguments.size;
			if( Len(arguments.align) )
				classes = classes & ' ' & arguments.align;
			if( Len(arguments.fontClasses) )
				classes = classes & ' ' & arguments.fontClasses;
			if( Len(arguments.effect) )
				classes = classes & ' ' & arguments.effect;
		
			// Link Classes
			if( Len(arguments.linkClass) )
				linkClasses = 'class="#arguments.linkClass#"';
				
			// Styles
			if( Len(arguments.color) )
			{
				if( len(arguments.linkURL) )
					linkStyleAttr = linkStyleAttr & 'color:#arguments.color#;';
				else
					styleAttr = styleAttr & 'color:#arguments.color#;';
			}
			if( Len(arguments.backgroundColor) )
				styleAttr = styleAttr & 'background-color:#arguments.backgroundColor#;';
			if( Len(arguments.padding) )
				styleAttr = styleAttr & 'padding:#arguments.padding#;';
			
			if( Len(styleAttr) )	
				styleAttr = 'style="#styleAttr#"';

			if( Len(linkStyleAttr) )
				linkStyleAttr = 'style="#linkStyleAttr#"';

			// onClick 
			if( Len(arguments.onClick) ) 
				onClickAttr = 'onclick="#trim(arguments.onClick)#"';
			
			// Link Attributes
			// - additional anchor tag attributes key/value pairs (eg. data-toggle="modal")
			for ( attrKey IN arguments.linkAttrStruct ) { 
				linkAttr = linkAttr & ' #attrKey#="#trim(arguments.linkAttrStruct[attrKey])#"';
			}

			if( Len(arguments.iconClass) and findNoCase('WithIcon',arguments.linkClass))
				theIcon = '<i #linkStyleAttr# class="#arguments.iconClass#" aria-hidden="true"></i>';
			
			// Linked or not linked?
			if( Len(arguments.linkURL) )
			{
				if( Len(arguments.iconClass) and findNoCase('WithIcon-left',arguments.linkClass) )
					theText = '<a href="#arguments.linkURL#" #linkClasses# #linkAttr# #linkStyleAttr# #onClickAttr#>#theIcon##theText#</a>';
				else if( Len(arguments.iconClass) and findNoCase('WithIcon-right',arguments.linkClass) )
					theText = '<a href="#arguments.linkURL#" #linkClasses# #linkAttr# #linkStyleAttr# #onClickAttr#>#theText##theIcon#</a>';	
				else
					theText = '<a href="#arguments.linkURL#" #linkClasses# #linkAttr# #linkStyleAttr# #onClickAttr#>#theText#</a>';	
			}				
		</cfscript>
		
		<cfif len(arguments.tag)>
			<cfoutput>
				<#arguments.tag# class="#classes#" #styleAttr#>#theText#</#arguments.tag#>
			</cfoutput>
		<cfelse>
			<cfoutput>#theText#</cfoutput>
		</cfif>
		
	</cffunction>

	<!----------------------------------------
		Render Line Core
	----------------------------------------->
	<cffunction name="renderLineCore" access="public" output="Yes" returntype="void">
		<cfargument name="class" type="string" required="Yes" hint="a class value">
		<cfargument name="color" type="string" required="Yes" hint="a style value">
		<cfargument name="backgroundColor" type="string" required="Yes" hint="a style value">
		<cfargument name="size" type="string" required="no" default="" hint="the border size (1px, etc)">
		<cfargument name="align" type="string" required="no" default="" hint="a class value">
		<cfargument name="padding" type="string" required="no" default="" hint="a style value">
		<cfargument name="effect" type="string" required="no" default="" hint="a class value">
		<cfargument name="style" type="string" required="no" default="" hint="a style value (includes width:{})">

		<cfscript>
			var classes = arguments.class;
			var styleAttr = '';
			var outerClasses = '';
		
			// Classes
			if( Len(arguments.align) )
				outerClasses = arguments.align;
			
			if( Len(arguments.effect) )
				classes = classes & ' ' & arguments.effect;
			
			// Styles
			if( Len(arguments.color) )
				styleAttr = styleAttr & 'width:#arguments.style#;';
			if( Len(arguments.color) )
				styleAttr = styleAttr & 'border-top-color:#arguments.color#;';
			if( Len(arguments.size) )
				styleAttr = styleAttr & 'border-top-width:#arguments.size#;';
			
			if( Len(styleAttr) )	
				styleAttr = 'style="#styleAttr#"';
		</cfscript>
	
		<cfoutput>
			<div class="cswLineWrapper #outerClasses#">
				<div class="cswLine #classes#" #styleAttr#></div>
			</div>
		</cfoutput>

	</cffunction>

	
	
	<!----------------------------------------
		Render Image
	----------------------------------------->
	<cffunction name="renderImage" access="public" output="Yes" returntype="void">
		<cfargument name="dataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
		<cfargument name="imgWrapperStyle" type="string" required="No" default="">
		<cfargument name="imgWrapperClass" type="string" required="No" default="">

		<cfscript>
			var data = arguments.dataStruct;
			var props = arguments.propsStruct;
			var styleAttr = '';
			var mainOverlayStyle = '';
			var hoverOverlayStyle  = '';
			var linkURL = '';
			var linkClasses = '';
			var linkStyleAttr = '';
			var onClickAttr = '';
			var linkAttr = '';
			var attrKey = '';
			
			var mainOverlayClasses = '';
			var hoverOverlayClasses = '';
			var wapperClasses = arguments.imgWrapperClass;
			
		
				
			// Image Movement	
			if( StructKeyExists(props,'ImageMovement') AND len(props.imageMovement) )
				wapperClasses = trim( wapperClasses & ' #props.imageMovement#');
		
			// Shading Styles
			if( StructKeyExists(props,'initialShading') AND len(props.initialShading.color) )
			{
				mainOverlayClasses = 'cswActive';
				mainOverlayStyle = 'color:#props.initialShading.color#;';
				if( len(props.initialShading.style) )
					mainOverlayClasses = trim( mainOverlayClasses & ' #props.initialShading.style#'); 	
				if( len(props.initialShading.effect) )
					mainOverlayClasses = trim( mainOverlayClasses & ' #props.initialShading.effect#'); 					
			}			

			if( StructKeyExists(props,'rolloverShading') AND len(props.rolloverShading.color) )
			{
				hoverOverlayClasses = 'cswActive';
				hoverOverlayStyle = 'color:#props.rolloverShading.color#;';
				if( len(props.rolloverShading.style) )
					hoverOverlayClasses = trim( hoverOverlayClasses & ' #props.rolloverShading.style#'); 	
				if( len(props.rolloverShading.effect) )
					hoverOverlayClasses = trim( hoverOverlayClasses & ' #props.rolloverShading.effect#'); 	
			}

			
			// configure other styles
			if( len(data.image) )
			{
				if( FindNoCase('percent', props.imageHeightType) )
				{
					styleAttr = styleAttr & 'background-image: url(#data.Image#); padding-bottom:#props.imageHeightPercentage#%;';
				}
				else if( FindNoCase('fixed', props.imageHeightType) )
				{
					styleAttr = styleAttr & 'background-image: url(#data.Image#); height:#props.imageHeightFixed#px;';
				}	
			}
			else
			{
				styleAttr = styleAttr & 'background-color: ##ccc; padding:100px 5px; display:inline-block; text-align: center;';
			}

			// Styles on Wrapper div	
			if( len(arguments.imgWrapperStyle) )
				arguments.imgWrapperStyle = 'style="#arguments.imgWrapperStyle#"';		
			
			// Styles on img			
			if( len(styleAttr) )
				styleAttr = 'style="#styleAttr#"';

			if( len(mainOverlayStyle) )
				mainOverlayStyle = 'style="#mainOverlayStyle#"';

			if( len(hoverOverlayStyle) )
				hoverOverlayStyle = 'style="#hoverOverlayStyle#"';
			
			// Link Porperties
			if( StructKeyExists(data,'LinkURL') )
				linkURL = data.LinkURL;

			// Link Classes
			if( StructKeyExists(props,'linkClass') AND Len(props.linkClass) )
				linkClasses = "#linkClasses# #props.linkClass#";
			
			if( Len(#linkClasses#) ) 	
				linkClasses = 'class="#linkClasses#"';
			
			// onClick 
			if( StructKeyExists(data,'onClick') AND Len(data.onClick) ) 
				onClickAttr = 'onclick="#trim(data.onClick)#"';
			
			// Link Attributes
			// - additional anchor tag attributes key/value pairs (eg. data-toggle="modal")
			if( StructKeyExists(data,'linkAttrStruct') )
			{
				for ( attrKey IN data.linkAttrStruct ) 
					linkAttr = linkAttr & ' #attrKey#="#trim(data.linkAttrStruct[attrKey])#"';
			}
		</cfscript>
		
		<cfoutput>
			<div class="cswImageWrapper #wapperClasses#" #arguments.imgWrapperStyle#>
		</cfoutput>
		
			<cfif LEN(TRIM(linkURL))>
				<cfoutput><a href="#LinkURL#" #linkClasses# #linkAttr# #onClickAttr#></cfoutput>
			</cfif>
			
			<cfif len(data.image) eq 0>
				<cfoutput>
					<div class="cswImage cswMissingImg" #styleAttr#>Missing Image</div>
				</cfoutput>
			<cfelseif FindNoCase('auto', props.imageHeightType)>
				<cfoutput>	
					<img src="#data.image#" class="cswImage" #styleAttr#>
				</cfoutput>
			<cfelse>
				<cfoutput>
					<div class="cswImage #data.imageFocalPoint#" #styleAttr#></div>
				</cfoutput>
			</cfif>
			
			<cfoutput>	
				<div class="cswMainImgOverlay #mainOverlayClasses#" #mainOverlayStyle#></div>
				<div class="cswHoverImgOverlay #hoverOverlayClasses#" #hoverOverlayStyle#></div>
			</cfoutput>	
				
		<cfif LEN(TRIM(linkURL))>		
			<cfoutput></a></cfoutput>
		</cfif>
		
		<cfoutput>	
			</div>
		</cfoutput>

	</cffunction>
	

	<!----------------------------------------
		Render Link/Button
	----------------------------------------->
	<cffunction name="renderButton" access="public" output="Yes" returntype="void">
		<cfargument name="dataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">

		<cfscript>
			var data = arguments.dataStruct;
			var props = arguments.propsStruct;
			var buttonIcon = '';
			var buttonCorners = '';
			var onClick = '';
			var linkAttrStruct = StructNew();
			
			var typeButtonType = 'cswNoButton';
			var typeLinkText = '';
			var typeLinkURL = '';
			var typeButtonCorners = '';
			var typeStruct = structNew();
			var typeIcon = '';
			var typeOnClick = '';
			var typeLinkAttrStruct = StructNew();
			

			if( StructKeyExists(data,'buttonType') AND len(data.buttonType) )
				typeButtonType = data.buttonType;
			else if( StructKeyExists(props,'buttonType') AND len(props.buttonType) )
				typeButtonType = props.buttonType;				
			
			if( typeButtonType eq 'cswNoButton')
				return;
			
			if( FindNoCase('cswButton', typeButtonType) GT 0 )
			{
				if( StructKeyExists(data,'button') AND NOT structIsEmpty(data.button) )
					typeStruct = data.button;
				else if( StructKeyExists(props,'button') )
					typeStruct = props.button;
			}
			else if( FindNoCase( 'cswLink', typeButtonType) GT 0 )
			{
				if( StructKeyExists(data,'link') AND NOT structIsEmpty(data.link) )
					typeStruct = data.link;
				else if( StructKeyExists(props,'link') )
					typeStruct = props.link;
			}

			if( StructKeyExists(data,'linkText') AND len(data.linkText) )
				typeLinkText = data.linkText;
			else if( StructKeyExists(props,'linkText') AND len(props.linkText) )
				typeLinkText = props.linkText;
				
			if( StructKeyExists(data,'linkURL') AND len(data.linkURL) )
				typeLinkURL = data.linkURL;
			else if( StructKeyExists(props,'linkURL') AND len(props.linkURL) )
				typeLinkURL = props.linkURL;
				
				
			if( len(typeLinkText) eq 0 OR len(typeLinkURL) EQ 0 )
				return;				
				
			if( StructKeyExists(data,'buttonCorners') AND len(data.buttonCorners) )
				typeButtonCorners = data.buttonCorners;
			else if( StructKeyExists(props,'buttonCorners') AND len(props.buttonCorners) )
				typeButtonCorners = props.buttonCorners;				
								
			if( StructKeyExists(data,'buttonIcon') AND len(data.buttonIcon) )
				typeIcon = data.buttonIcon;
			else if( StructKeyExists(props,'buttonIcon') AND len(props.buttonIcon) )
				typeIcon = props.buttonIcon;

			if( StructKeyExists(data,'onClick') AND len(data.onClick) )
				typeOnClick = data.onClick;
			else if( StructKeyExists(props,'onClick') AND len(props.onClick) )
				typeOnClick = props.onClick;

			if( StructKeyExists(data,'linkAttrStruct') )
				typeLinkAttrStruct = data.linkAttrStruct;
			else if( StructKeyExists(props,'linkAttrStruct') )
				typeLinkAttrStruct = props.linkAttrStruct;
				
/*
			tmp = StructNew();
			tmp.buttontype=typeButtonType;
			tmp.linkText=typeLinkText;
			tmp.linkURL=typeLinkURL;
			tmp.buttonStruct=typeStruct;
			tmp.buttonCorners=typeButtonCorners;
			tmp.icon=typeIcon;
			tmp.onClick=typeOnClick;
			tmp.linkAttrStruct=typeLinkAttrStruct;
			// writeDump(data);
			// writeDump(var=tmp, expand='yes');
*/
					
			renderButtonCore( type = typeButtonType,
									linkText = typeLinkText,
									linkURL = typeLinkURL,
									buttonStruct = typeStruct,
									linkStruct = typeStruct,
									buttonCorners = typeButtonCorners,
									icon = typeIcon,
									onClick = typeOnClick,
									linkAttrStruct = typeLinkAttrStruct);
		</cfscript>
	</cffunction>

	
	<!----------------------------------------
		Render Link/Button Core
		
		ButtonStruct can contain these fields:
		
			// styles
			color
			bgColor
			style
			padding
			
			// classes
			align	
			effect
			fontClasses
			size
			style
			
			// Other
			wrapperStyles
			wrapperClasses
			
	----------------------------------------->
	<cffunction name="renderButtonCore" access="public" output="Yes" returntype="void">
		<cfargument name="type" type="string" required="Yes">
		<cfargument name="linkText" type="string" required="No" default="">
		<cfargument name="linkURL" type="string" required="No" default="">
		<cfargument name="buttonStruct" type="struct" required="No" default=#StructNew()#>
		<cfargument name="linkStruct" type="struct" required="No" default=#StructNew()#>
		<cfargument name="buttonCorners" type="string" required="No" default="">
		<cfargument name="icon" type="string" required="No" default="">
		<cfargument name="onClick" type="string" required="No" default="">
		<cfargument name="linkAttrStruct" type="struct" required="No" default=StructNew()>

	
		<cfscript>
			// var data = arguments.dataStruct;
			// var props = arguments.propsStruct;
			var button = arguments.buttonStruct;
			var link = arguments.linkStruct;
			var styleAttr = '';
			var DataAttr = '';
			var innerStyleAttr = '';
			var iconStyleAttr = '';
			var classes = '';
			var onClickAttr = '';
			var linkAttr = '';
			var attrKey = '';
			var linkStyle = '';
		
			var keys = 'align,padding,fontClasses,size,effect,style,color,bgColor,wrapperClasses,wrapperStyles';

			// Set Link/Button Props Defaults
			for( i=1; i lte ListLen(keys); i=i+1 )
			{
				key = listGetAt(keys, i);
				if( ! StructKeyExists( button, key ) )
					button[key] = '';
				if( ! StructKeyExists( link, key ) )
					link[key] = '';
			}

			if( arguments.type neq 'cswNoButton' )
			{
				// BUTTON 
				// -----------
				if( FindNoCase('cswButton', arguments.type) GT 0 )
				{
					if( len(button.color) )
						innerStyleAttr = innerStyleAttr & 'color:#button.color#;';
						
					iconStyleAttr = innerStyleAttr;
					
					if( len(button.bgColor) )
					{
						styleAttr = styleAttr & 'background-color:#button.bgColor#;';
						if( FindNoCase("cswBtn-outlined", button.style) AND len(button.color) )
							styleAttr = styleAttr & 'color:#button.color#;';
					}
					if( len(button.padding) )
						styleAttr = styleAttr & 'padding:#button.padding#;';
						
					if( len(button.wrapperStyles) )
						button.wrapperStyles = 'style="#button.wrapperStyles#";';
						
					if( len(styleAttr) )
						styleAttr = 'style="#styleAttr#"';
					if( len(innerStyleAttr) )
						innerStyleAttr = 'style="#innerStyleAttr#"';
					if( len(iconStyleAttr) )
						iconStyleAttr = 'style="#iconStyleAttr#"';

					if( len(button.fontClasses) )
						classes = trim(classes & ' #button.fontClasses#');
					if( len(button.size) )
						classes = trim(classes & ' #button.size#');
					if( len(button.effect) )
						classes = trim(classes & ' #button.effect#');
					if( len(button.style) )
						classes = trim(classes & ' #button.style#');
					if( len(arguments.buttoncorners) )
						classes = trim(classes & ' #arguments.buttonCorners#');
						
					// onClick 
					if( len(arguments.onClick) ) 
						onClickAttr = 'onclick="#trim(arguments.onClick)#"';
			
					// Link Attributes
					// - additional anchor tag attributes key/value pairs (eg. data-toggle="modal")
					for( attrKey IN arguments.linkAttrStruct ) 
							linkAttr = linkAttr & ' #attrKey#="#trim(arguments.linkAttrStruct[attrKey])#"';
				
					// Render the Button					
					if( len(arguments.linkURL) AND len(arguments.linkText) )
					{
						WriteOutput('<div class="cswBtnWrapper #button.align# #button.wrapperClasses#" #button.wrapperStyles#>' );
						
						WriteOutput('<a href="#arguments.linkURL#" class="cswBtn #arguments.type# #classes#" #styleAttr# role="button" #linkAttr# #onClickAttr#>');
				
						if( FindNoCase('withIcon-left', arguments.type) GT 0 AND len(arguments.icon))
							WriteOutput('<i #iconStyleAttr# class="#arguments.icon#" aria-hidden="true"></i>');
				
						if( len(arguments.linkText) )
							WriteOutput('<span #innerStyleAttr#>#arguments.linkText#</span>' );

						if( FindNoCase('withIcon-right', arguments.type) GT 0 AND len(arguments.icon))
							WriteOutput('<i #iconStyleAttr# class="#arguments.icon#" aria-hidden="true"></i>');
						
						WriteOutput('</a>');
						WriteOutput('</div>' );
					}

				}
				else // LINK
				{
					// If link is not passed, use button style
					if( StructIsEmpty(link) ){
						link = button;
					}

					linkStyle = link.style;
					if ( len(link.wrapperStyles) )
						linkStyle = linkStyle & link.wrapperStyles;
					
					RenderTextCore(
						class = 'cswLinkWrapper #link.wrapperClasses#',
						tag = 'div',
						text = arguments.linkText,
						color = link.color,
						backgroundColor = link.bgColor,
						size = link.size,
						align = link.align,
						fontClasses = link.fontClasses,
						padding = link.padding,
						effect = link.effect,
						linkURL = arguments.linkURL,
						linkClass = arguments.type,
						iconClass = arguments.icon,
						style = linkStyle,
						onClick = arguments.onClick,
						linkAttrStruct = arguments.linkAttrStruct );
				}
			}
		</cfscript>
	
	</cffunction>

	
	<!--- ------------------------------------------ --->
	<!---        Custom Field Data Processing        --->
	<!--- ------------------------------------------ --->

	<!----------------------------------------
		Get Field Settings
			- from the JSON string for this field from the widgetdata struct
	----------------------------------------->
	<cffunction name="getFieldSettings" access="public" output="No" returntype="struct">
		<cfargument name="widgetData" type="struct" required="Yes">
		<cfargument name="field" type="string" required="Yes">
		<cfargument name="defShow" type="string" required="No" default="no">
		<cfargument name="defTag" type="string" required="No" default="p">
		<cfargument name="defColor" type="string" required="No" default="">
		<cfargument name="defBgColor" type="string" required="No" default="">
		<cfargument name="defBorderColor" type="string" required="No" default="">
		<cfargument name="defSize" type="string" required="No" default="">
		<cfargument name="defAlign" type="string" required="No" default="">
		<cfargument name="defStyle" type="string" required="No" default="">
		<cfargument name="defEffects" type="string" required="No" default="">
		<cfargument name="defPadding" type="string" required="No" default="">
		<cfargument name="defFontSettings" type="string" required="No" default="bold:0,italic:0,underline:0,case:none">
	
		<cfscript>
			var props = StructNew();

			props[arguments.field] = StructNew();
			
			props[arguments.field].show = arguments.defShow;
			props[arguments.field].tag = arguments.defTag;
			props[arguments.field].color = arguments.defColor;
			props[arguments.field].bgColor = arguments.defBgColor;
			props[arguments.field].borderColor = arguments.defBorderColor;
			props[arguments.field].size = arguments.defSize;
			props[arguments.field].align = arguments.defAlign;			
			props[arguments.field].style = arguments.defStyle;
			props[arguments.field].effect = arguments.defEffects;			
			props[arguments.field].padding = arguments.defPadding;
			props[arguments.field].fontSettings = arguments.defFontSettings;
			
			if( StructKeyExists(arguments.widgetData, arguments.field) 
						AND arguments.widgetData[arguments.field] neq '' 
						AND isJSON(arguments.widgetData[arguments.field]) )
			{			
				temp = DeserializeJSON(arguments.widgetData[arguments.field]);
				
				StructAppend( props[arguments.field], temp, true );
			}
			
			// convert font properties into class list				
			props[arguments.field].fontClasses = getFontClasses( props[arguments.field].fontSettings );

			return props[arguments.field];		
		</cfscript>
	
	</cffunction>
	
	<!----------------------------------------
		Get Data Field Settings 
			- from the JSON string for this field from the widgetdata struct
			- returns empty struct if data field is not mapped
	----------------------------------------->
	<cffunction name="getDataFieldSettings" access="public" output="No" returntype="struct">
		<cfargument name="widgetData" type="struct" required="Yes">
		<cfargument name="field" type="string" required="Yes">
		<cfargument name="defShow" type="string" required="No" default="no">
		<cfargument name="defTag" type="string" required="No" default="p">
		<cfargument name="defColor" type="string" required="No" default="">
		<cfargument name="defBgColor" type="string" required="No" default="">
		<cfargument name="defBorderColor" type="string" required="No" default="">
		<cfargument name="defSize" type="string" required="No" default="">
		<cfargument name="defAlign" type="string" required="No" default="">
		<cfargument name="defStyle" type="string" required="No" default="">
		<cfargument name="defEffects" type="string" required="No" default="">
		<cfargument name="defPadding" type="string" required="No" default="">
		<cfargument name="defFontSettings" type="string" required="No" default="bold:0,italic:0,underline:0,case:none">
	
		<cfscript>
			var props = StructNew();
			
			if( NOT StructKeyExists(arguments.widgetData, arguments.field) )
				return props;	
				
			return getFieldSettings( argumentCollection = arguments );
		</cfscript>
		
	</cffunction>	
	
	<!----------------------------------------
		Get Font Classes
			- returns a string of proper css classes
	----------------------------------------->	
	<cffunction name="getFontClasses" access="public" returntype="string" output="No">
		<cfargument name="fontSettings" type="string" required="yes">
		
		<cfscript>
			var classes = '';
			var str = StructNew();
			var i = 0;
			var keys = 'bold,italic,underline,case';
			var item = '';
			var name = '';
			var value = '';
			
			str['bold'] = 0;			
			str['italic'] = 0;			
			str['underline'] = 0;			
			str['case'] = 'none';			
			
			for( i=1; i lte listLen(arguments.fontSettings); i=i+1 )
			{
				item = ListGetAt(arguments.fontSettings, i);
				name = ListFirst(item, ':');
				value = ListLast(item, ':');
				
				if( FindNoCase( name, keys) gt 0 )
				{
					// valid key
					str[name] = value;
				}
			}
				
			if( str.bold eq 1 )
				classes = trim(classes & ' cswTextBold');		
			if( str.italic eq 1 )
				classes = trim(classes & ' cswTextItalic');		
			if( str.underline eq 1 )
				classes = trim(classes & ' cswTextUnderline');		
			if( str.case neq 'none' )
				classes = trim(classes & ' cswText-#LCase(str.case)#');		
				
			return classes;	
		</cfscript>
	
	</cffunction>

	
	<!----------------------------------------
		Get Device Size Options Struct
			- from the JSON string for this field from the widgetdata struct
	----------------------------------------->
	<cffunction name="getDeviceSizeOptions" access="public" output="No" returntype="Struct">
		<cfargument name="widgetData" type="struct" required="Yes">
		<cfargument name="field" type="string" required="Yes">
		<cfargument name="defXXS" type="string" required="No" default="p">
		<cfargument name="defXS" type="string" required="No" default="">
		<cfargument name="defSM" type="string" required="No" default="">
		<cfargument name="defMD" type="string" required="No" default="">
		<cfargument name="defLG" type="string" required="No" default="">
	
		<cfscript>
			var props = StructNew();
		
			props[arguments.field] = StructNew();
			if( StructKeyExists(arguments.widgetData, arguments.field) 
						AND arguments.widgetData[arguments.field] neq '' AND isJSON(arguments.widgetData[arguments.field]) )
			{			
				props[arguments.field] = DeserializeJSON(arguments.widgetData[arguments.field]);
			}
			else
			{
				props[arguments.field].xxs = arguments.defXXS;
				props[arguments.field].xs = arguments.defXS;
				props[arguments.field].sm = arguments.defSM;
				props[arguments.field].md = arguments.defMD;
				props[arguments.field].lg = arguments.defLG;			
			}
			
			return props[arguments.field];	
		</cfscript>
	
	</cffunction>
	
	<!----------------------------------------
		Alter Column Classes 
			- Alter columns item quanity is less that select columns sizes
	----------------------------------------->
	<cffunction name="alterColumnClasses" access="public" output="No" returntype="struct">
		<cfargument name="itemCount" type="numeric" required="Yes">
		<cfargument name="columnStruct" type="struct" required="Yes">
		
		<cfscript>
			var cKey = "";
			var colClass = "";
			var colCount = 0;
			var props = StructNew();
			
			// Build the Column classes (compare column count to item count)
			for ( cKey IN arguments.columnStruct ) {
				colClass = arguments.columnStruct[cKey];
				colCount = ListLast(colClass,"-");
				if ( arguments.itemCount GT 0 and IsNumeric(colCount) and arguments.itemCount LT colCount )
					colClass = Replace(colClass, "-" & colCount, "-" & arguments.itemCount, "one");
					
				props[cKey] = colClass;
			}
		
			return props;	
		</cfscript>
	</cffunction>

	<!----------------------------------------
		Get Device Size Class List String 
			- from the Device Size Options Struct 
	----------------------------------------->
	<cffunction name="getDeviceSizeClassList" access="public" output="No" returntype="string">
		<cfargument name="deviceData" type="struct" required="Yes">
		<cfargument name="delimiter" type="string" required="No" default=" ">
	
		<cfscript>
			var cKey = "";
			var propsStr = "";
	
			// Build the Column classes
			for ( cKey IN arguments.deviceData ) {
				propsStr = ListAppend(propsStr, arguments.deviceData[cKey], "#arguments.delimiter#");
			}
		
			return propsStr;	
		</cfscript>
	
	</cffunction>

	<!----------------------------------------
		Get Standard Field Properties String
			- from the widgetdata struct
	----------------------------------------->
	<cffunction name="getStandardField" access="public" output="No" returntype="string">
		<cfargument name="widgetData" type="struct" required="Yes">
		<cfargument name="field" type="string" required="Yes">
		<cfargument name="defValue" type="string" required="No" default="">
	
		<cfscript>
			var props = '';

			if( StructKeyExists(arguments.widgetData, arguments.field) )
				props = arguments.widgetData[arguments.field];
			else
				props = arguments.defValue;

			return trim(props);		
		</cfscript>
	
	</cffunction>
	
	
	<!----------------------------------------
		Get Standard Metadata Image Field Properties String
			- from the widgetdata struct
	----------------------------------------->
	<cffunction name="getStandardMetadataImageField" access="public" output="No" returntype="string">
		<cfargument name="widgetData" type="struct" required="Yes">
		<cfargument name="field" type="string" required="Yes">
		<cfargument name="defValue" type="string" required="No" default="">
	
		<cfscript>
			var img = '';
			var imgID = 0;	
			var imgObj = server.commonspot.api.getObject('image');
			var imgInfo = StructNew();	
			var props = '';		

			if( StructKeyExists(arguments.widgetData, arguments.field) )
			{
				img = arguments.widgetData[arguments.field];
				
				if( img neq '')
				{
					imgID = ListLast( ListFirst(img, '|'), ':' );
					imgObj = server.commonspot.api.getObject('image');
					imgInfo = imgObj.getInfo( imgID );
					
					img = imgInfo.serverURL;
				}
				
				props = img;
				
			}
			else
				props = arguments.defValue;

			return trim(props);		
		</cfscript>
	
	</cffunction>
	

	<!----------------------------------------
		Get Standard Custom Element Image Field Properties String
			- from the widgetdata struct
	----------------------------------------->
	<cffunction name="getStandardCustomElementImageField" access="public" output="No" returntype="string">
		<cfargument name="widgetData" type="struct" required="Yes">
		<cfargument name="field" type="string" required="Yes">
		<cfargument name="defValue" type="string" required="No" default="">
	
		<cfscript>
			var img = '';	
			var data = '';	

			if( StructKeyExists(arguments.widgetData, arguments.field) )
			{
				img = arguments.widgetData[arguments.field];

				if( isStruct(img) AND StructKeyExists(img,'ResolvedURL') )
				{
					if( img.ResolvedURL.serverRelative neq '' )
						img = img.ResolvedURL.serverRelative;
					else
						img = img.ResolvedURL.Absolute;
				}
				
				data = img;
			}
			else
				data = arguments.defValue;

			return trim(data);		
		</cfscript>
	
	</cffunction>



	<!----------------------------------------
		Get Standard Custom Element Link Field Properties String
			- from the widgetdata struct
	----------------------------------------->
	<cffunction name="getStandardCustomElementLinkField" access="public" output="No" returntype="string">
		<cfargument name="widgetData" type="struct" required="Yes">
		<cfargument name="field" type="string" required="Yes">
		<cfargument name="defValue" type="string" required="No" default="">
	
		<cfscript>
			var link = '';
			var data = '';
			
			// Check if '_csURL' is on field Name, if so remove
			if( FindNoCase('_csURL', arguments.field ) )
				arguments.field = Replace( arguments.field, '_csURL', '' );

			// Check if bound to STRUCT		
			if( StructKeyExists(arguments.widgetData, arguments.field & '_csURL') )
			{
				link = arguments.widgetData[arguments.field & '_csURL'];

				if( StructKeyExists(link,'ResolvedURL') )
				{
					if( Len(trim(link.ResolvedURL.serverRelative)) )
						link = link.ResolvedURL.serverRelative;
					else
						link = link.ResolvedURL.Absolute;
				}
				
				data = link;
			}
			// NO Struct - Straight URL
			else if( StructKeyExists(arguments.widgetData, arguments.field) )
			{
				data = arguments.widgetData[arguments.field];
			}
			else
				data = arguments.defValue;

			return trim(data);		
		</cfscript>
	
	</cffunction>


	<!----------------------------------------
		hasPropsLayoutBinding() 
	----------------------------------------->
	<cffunction name="hasPropsLayoutBinding" access="public" returntype="boolean" output="Yes">
		<cfargument name="ElementInfo" type="struct" required="true">
		<cfargument name="MetadataFormName" type="string" required="true">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">
		
		<cfscript>
			var errMsg = '';
			var rhFormBindingQry = '';
			var mdCheckQry = '';
			var mdQry = '';
			var mdResultQry = QueryNew('tmp');
			var errMsg = "Please bind the '#arguments.metadataFormName#' Metadata Form to this Layout!";

			if( NOT StructIsEmpty(arguments.rhCaller) )
			{
				// Render handler Custom Metadata Form Binding Error handling
				if( NOT StructKeyExists(arguments.rhCaller,"handlerforms") OR NOT IsQuery(arguments.rhCaller.handlerforms) OR arguments.rhCaller.handlerforms.RecordCount EQ 0 )
				{
					if( request.user.id gt 0 AND server.commonspot.udf.security.authorok() == 1 )
						WriteOutput('<div><em>#errMsg#</em></div>');

					return false;
				}
				else
				{
					 // Set the main RH Form Bind Query
					 rhFormBindingQry = arguments.rhCaller.handlerforms;

					 // Query of Queries
					 mdQry = new Query();
					 mdQry.setDBType('query');
					 mdQry.setAttributes(mdCheckQry=rhFormBindingQry); 	// needed for QoQ
					 mdQry.addParam(name='FormName', value='#Trim(Lcase(arguments.MetadataFormName))#', cfsqltype='cf_sql_varchar');
					 mdQry.setSQL('SELECT FormName FROM mdCheckQry where lower(FormName) = :FormName');
					 //mdQry.setSQL('SELECT * FROM mdCheckQry');
					 //mdQry.setMaxRows(1); 							// limit max rows
					 mdResultQry = mdQry.execute().getResult();
				}

				// Check if the RH has specific Metadata form
				if( NOT mdResultQry.RecordCount )
				{
					if ( request.user.id gt 0 AND server.commonspot.udf.security.authorok() == 1 )
						WriteOutput('<div><em>#errMsg#</em></div>');

					return false;
				}
			}
			
			if( NOT StructKeyExists(arguments.ElementInfo,"RenderHandlerMetaData") 
				 OR NOT StructKeyExists(arguments.ElementInfo.RenderHandlerMetaData, arguments.MetadataFormName )
				 OR StructIsEmpty(arguments.ElementInfo.RenderHandlerMetaData[arguments.MetadataFormName]) )
			{
				// Props data Missing Error Message
				errMsg = "No properties configured. Please open this element's menu and choose the 'Custom' menu option, then configure and save the dialog.";
				if ( arguments.showDataError AND request.user.id gt 0 AND server.commonspot.udf.security.authorok() == 1 )
					WriteOutput('<div><em>#errMsg#</em></div>');
				
				return false;
			}

			return true;
		</cfscript>
		
	</cffunction>
	
	
	<!----------------------------------------
		hasProperties()
	----------------------------------------->
	<cffunction name="hasProperties" access="public" returntype="boolean" output="Yes">
		<cfargument name="propsStruct" type="struct" required="false" default="#StructNew()#">

		<cfscript>
			var errMsg = '';
			var props = arguments.propsStruct;
			
			if( StructIsEmpty(props) )
				return false;
			
			return true;
		</cfscript>

	</cffunction>
	
	
	
	<!----------------------------------------
		getRGBAColor()
	----------------------------------------->
	<cffunction name="getRGBAColor" access="public" returntype="string" output="Yes">
		<cfargument name="color" type="string" required="true">
		<cfargument name="opacity" type="string" required="true">

		<cfscript>
			var c = trim(arguments.color);
			var o = trim(arguments.opacity);
			var pos = FindNoCase( 'rgb', c );

			// if the passed color does not contain 'rgb' return what was passed in
			if( pos NEQ 1 )
				return c;
				
			if( FindNoCase( 'rgba(', c ) GT 0 )
				c = replaceNoCase( c, 'rgba(', '', 'all' );
			else if( FindNoCase( 'rgb(', c ) GT 0 )
				c = replaceNoCase( c, 'rgb(', '', 'all' );
			c = replaceNoCase( c, ')', '', 'all' ); 

			// if the passed color does not contain 3 colors, return what was passed in
			if( ListLen(c) NEQ 3 )
				return c;
								
			if( NOT isNumeric(o) )
				o = '1.0';
			
			return 'rgba(#c#,#o#)';
		</cfscript>

	</cffunction>
	
	

</cfcomponent>
