<!--- 
	csw-tabs-common.cfc 
	
	Version: [2018-01-02 01:28 pm]
--->


<cfcomponent displayname="csw-tabs-common" output="false" hint="Common Tabs Functions" extends="_cs_widgets._common.csw-common">

	<cffunction name="loadResources" access="public" output="Yes" returntype="void">
		<cfargument name="mainResource" type="string" required="Yes">
		
		<cfscript>
			Server.CommonSpot.udf.resources.loadResources("jQuery");
			Server.CommonSpot.udf.resources.loadResources("FontAwesome");
			Server.CommonSpot.udf.resources.loadResources("#arguments.mainResource#");
			Server.CommonSpot.udf.resources.loadResources("cswCommonStyles");
			Server.CommonSpot.udf.resources.loadResources("cswTextStyles");

			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswCommonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswCommonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswTextStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswTextStyles_custom");
				
			if ( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'#arguments.mainResource#_custom') )
				Server.CommonSpot.udf.resources.loadResources("#arguments.mainResource#_custom");
		</cfscript>
	</cffunction>

	
	<cffunction name="loadCommonProperties" access="public" returntype="struct" output="No">
		<cfargument name="metadataFormName" type="string" required="true">
		<cfargument name="elementInfo" type="struct" required="Yes">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">
		
		<cfscript>
			var widgetMetadataFormName = arguments.metadataFormName;
			var props = StructNew();
			var rhMetadata = arguments.ElementInfo.RenderHandlerMetaData;
			var widgetData = StructNew();

			widgetData = rhMetadata[widgetMetadataFormName];			
			
			/********************************************/
			/**** TAB & CONTENT PROPERTIES TAB **********/
			/********************************************/

			// Tab Type 
			props.tabType = getStandardField( widgetData=widgetData, field='Tab_Type', defValue='nav-tabs' );

			// Tab Bar Style
			props.tabBar = getFieldSettings( widgetData=widgetData, field='Tab_Bar' );		
				
			// Content Area
			props.contentArea = getFieldSettings( widgetData=widgetData, field='Content_Area' );			
			

			/************************************/
			/**** DEFAULT TAB PROPERTIES TAB ****/
			/************************************/
			
			// Default Style
			props.defaultTab = getFieldSettings(widgetData=widgetData, field='Default_Style' );
			
			// Default Padding
			props.defaultPadding = getStandardField(widgetData=widgetData, field='Tab_Padding' );

			// Border Style
			props.defaultBorder = getFieldSettings(widgetData=widgetData, field='Border_Style' );
			
			// Corners 
			props.defaultCorners = getStandardField(widgetData=widgetData, field='Corners' );
			
			// show Icon
			props.showIcon = getStandardField(widgetData=widgetData, field='Show_Icon' );
			
			
			/************************************/
			/**** ACTIVE TAB PROPERTIES TAB ****/
			/************************************/

			// Active Style
			props.activeTab = getFieldSettings(widgetData=widgetData, field='Active_Tab_Style' );
			
			// Active Border Style
			props.activeBorder = getFieldSettings(widgetData=widgetData, field='Active_Border_Style' );

			// Hightlight Line (borderColor & size)
			props.highlightLine = getFieldSettings(widgetData=widgetData, field='Highlight_line' );
			
			
			/************************************/
			/**** HOVER TAB PROPERTIES TAB ****/
			/************************************/

			// Hover Style
			props.hoverTab = getFieldSettings(widgetData=widgetData, field='Hover_Tab_Style' );
			
			// Hover Border Style
			props.hoverBorder = getFieldSettings(widgetData=widgetData, field='Hover_Border_Style' );
						
			
			/**************************************/
			/**** GENERAL PROPERTIES TAB **********/
			/**************************************/

			// Element (STRUCT)
			props.element = getFieldSettings(widgetData=widgetData, field='Overall' );

			// Background Image (STRUCT)
			props.backgroundImage = getStandardMetadataImageField( widgetData=widgetData, field='Background_Image', defValue='' );

			// Background Image Focal Point
			props.backgroundImageFocalPoint = getStandardField(widgetData=widgetData, field='Background_Image_Focal_Point', defValue='cswImgPos-centermiddle');

			return props;
		</cfscript>
	</cffunction>
	
	

	<cffunction name="renderTabContainers" access="public" output="Yes" returntype="string">
		<cfargument name="tabArray" required="Yes" type="array">

		<cfscript>
			var data = arguments.tabArray;
			var tabContainerHTML = "";
			var contentActiveClass = "";
			var ariaHiddenState = "true";
			var idx = 0;
		</cfscript>

		<cfsavecontent variable="tabContainerHTML">

			<!--- // Tab Content --->
			<cfoutput>
			<div class="cswContentWrap">
			</cfoutput>

				<cfloop index="idx" from="1" to="#arraylen(data)#">

					<cfscript>
						contentActiveClass = "";
						ariaHiddenState = "true";
						if ( idx EQ 1 )
						{
							ariaHiddenState = "false";
							contentActiveClass = "content-current";
						}
					</cfscript>

					<cfoutput>
					<section id="#data[idx].tabKey#" aria-hidden="#ariaHiddenState#" role="tabpanel" aria-labelledby="#data[idx].tabKey#"
						class="#contentActiveClass# cswTabContent" data-title="#XmlFormat(data[idx].title)#">
					</cfoutput>	

						<!--- // Render a container for each tab --->
						<CFMODULE TEMPLATE="/commonspot/utilities/ct-render-named-element.cfm"
									elementtype="container"
									elementName="tabContainer_#data[idx].tabID#_#request.element.id#">
									
					<cfoutput>	
					</section>
					</cfoutput>
					
				</cfloop>

			<cfoutput>	
			</div> <!--- // END - Tab Content Column --->
			</cfoutput>

		</cfsavecontent>

		<cfreturn tabContainerHTML>

	</cffunction>
	
	
	
	<cffunction name="renderTabs" access="public" output="Yes" returntype="array">
		<cfargument name="dataStruct" required="Yes" type="struct">
		<cfargument name="propsStruct" required="Yes" type="struct">

		<cfscript>
			var dStruct = arguments.dataStruct.propertyvalues;
			var props = arguments.propsStruct;			
			var numData = arraylen(dStruct);
			var elementID = request.element.id;
			var tabCounter = 0;
			var idx = 0;
			var tabTitleID = 0;
			var tabID = 0;
			var tabHashID = 0;
			var tabKey = 0;
			var tabContentKey = 0;
			var tabActiveClass = '';
			var tabArray = arrayNew(1);
			var tabBarStyleAttr = '';
			var tabBarClasses = '';	
			var data = StructNew();
			var textClasses = '';	
			
			
			// Tab Bar  styles & classes
			if( len(props.tabBar.padding) )
				tabBarStyleAttr = tabBarStyleAttr & "padding: #props.tabBar.padding#;";
			if( len(props.tabBar.bgColor) )
				tabBarStyleAttr = tabBarStyleAttr & "background-color: #props.tabBar.bgColor#;";
				
			if( len(tabBarStyleAttr) )	
				tabBarStyleAttr = 'style="#tabBarStyleAttr#"';
			
			if( len(props.tabBar.align) )
				tabBarClasses = trim( tabBarClasses & ' #props.tabBar.align#');	
		</cfscript>


		<!--- // Tab Bar --->
		<cfoutput>
		<nav class="cswTabBar #tabBarClasses#" #tabBarStyleAttr#>
			<ul>
			</cfoutput>
	
			<cfloop index="idx" from="1" to="#numData#">
	
				<cfif structKeyExists(dStruct[idx].values,"title") and len(dStruct[idx].values.title)>
	
					<cfscript>
						data = StructNew();
						
						// Get TabID (Optional)
						// if it does NOT exists, then tabs are bound to content by their ORDER (Ugh!)
						if( StructKeyExists(dStruct[idx].values, 'tabID') )
							data.tabID = dStruct[idx].values.tabID;
						else
							data.tabID = idx;
							
						data.title = dStruct[idx].values.title;
						if( structKeyExists( dStruct[idx].values, 'icon') )
							data.icon = dStruct[idx].values.icon;
						else
							data.icon = '';
						
						tabCounter = tabCounter + 1;
	
						// Build the simple tabTitleID
						tabTitleID = LCase( reReplace( application.ADF.data.unescapeHTMLentities(data.title), '[^a-zA-Z0-9\-\_]', '-', 'ALL' ) );
						tabTitleID = REReplaceNoCase(tabTitleID,'-{2,}','-','all');
						tabTitleID = ListChangeDelims(tabTitleID, "-", "-");
	
						// Build the unique TabID & Set the tab DIV IDs
						tabID = tabTitleID & "-" & elementID & "-" & data.tabID;
						tabHashID = 'hTabID-' & tabTitleID;
						tabKey = "hTab-" & tabID;
						tabContentKey = "hTabContent-" & tabID;
	
						tabActiveClass = '';
						if( tabCounter EQ 1 )
							tabActiveClass = "tab-current";
							
						// text size
						if( len(props.defaultTab.size) )
							textClasses = props.defaultTab.size;
	
						// fontClasses
						props.defaultTab.fontClasses = getFontClasses( props.defaultTab.fontSettings );							
					</cfscript>
						
					<!--- Render the Tab --->	
					<cfoutput>
						<li role="presentation" class="#tabActiveClass# cswTabNavItem #props.defaultTab.fontClasses# #textClasses# #props.showIcon#">
							<a href="###tabKey#" id="#tabHashID#">					
					</cfoutput>
	
						<cfif props.tabType neq 'cswTabs-folders'>
							<cfoutput>
								<div class="cswInner">
									<cfif len(data.icon) AND (props.showIcon eq 'cswTabIconAlign-left' OR props.showIcon eq 'cswTabIconAlign-top')>
										<i class="#data.icon# cswTabIcon"></i>
									</cfif>
									<span class="cswTabTitle">#data.title#</span>
									<cfif len(data.icon) AND props.showIcon eq 'cswTabIconAlign-right'>
										<i class="#data.icon# cswTabIcon"></i>
									</cfif>
								</div>
							</cfoutput>
							
						<cfelse>		<!---// FOLDERS --->
					
							<cfoutput>
								<cfif idx eq 1 OR idx eq numData>
									<svg viewBox="0 0 80 60" preserveAspectRatio="none"><use xlink:href="##tabshape"></use></svg>
								<cfelse>
									<svg viewBox="0 0 80 60" preserveAspectRatio="none"><use xlink:href="##tabshape"></use></svg>
									<svg viewBox="0 0 80 60" preserveAspectRatio="none"><use xlink:href="##tabshape"></use></svg>
								</cfif>
								
								<div class="cswInner">	
									<cfif len(data.icon) AND (props.showIcon eq 'cswTabIconAlign-left' OR props.showIcon eq 'cswTabIconAlign-top')>
										<i class="#data.icon# cswTabIcon"></i>
									</cfif>
									<span class="cswTabTitle">#data.title#</span>
									<cfif len(data.icon) AND props.showIcon eq 'cswTabIconAlign-right'>
										<i class="#data.icon# cswTabIcon"></i>
									</cfif>
								</div>	
							</cfoutput>	
						</cfif>

					<cfoutput>
						</a>
					</li>
					</cfoutput>
						
					<cfscript>
						tabStruct = structNew();
						tabStruct.tabKey = tabKey;		// DOM ID
						tabStruct.tabContentKey = tabContentKey;
						tabStruct.title = data.title;
						tabStruct.tabID = data.tabID;	// unique ID, or the tab position if tabID not passed
	
						arrayAppend(tabArray,tabStruct);
					</cfscript>
	
				</cfif>
	
			</cfloop>
		
			<cfoutput>	
			</ul> 
		</nav>
		</cfoutput>
		
		<cfreturn tabArray>

	</cffunction>
	
		


	<cffunction name="GetFontStyles" access="private" output="No" returntype="string">
		<cfargument name="fontSettings" type="string" required="yes">
		
		<cfscript>
			var st = StructNew();
			var key = '';
			var value = '';
			var item = '';
			var retString = '';

			for( i=1; i lte ListLen(arguments.FontSettings); i++ )
			{
				item = ListGetAt(arguments.FontSettings, i);
				key = ListFirst(item,':');
				value = ListLast(item,':');
				st[key] = value;
			}	
			
			// Bold
			if( structKeyExists(st, 'bold') )
			{
				if( st.bold eq 1 )
					retstring = 'font-weight:bold;';
			}

			// Italic
			if( structKeyExists(st, 'italic') )
			{
				if( st.italic eq 1 )
					retstring = retstring & ' font-style:italic;';
			}
			
			// Underline			
			if( structKeyExists(st, 'underline') )
			{
				if( st.underline eq 1 )
					retstring = retstring & ' text-decoration:underline;';
			}			
			
			// Case
			if( structKeyExists(st, 'case') )
			{
				retstring = retstring & ' text-transform:#st.case#;';
			}											
			
			return retString;
		</cfscript>
		
	</cffunction>
	
	
	<cffunction name="getBorderStyles" access="private" output="No" returntype="struct">
		<cfargument name="default" type="struct" required="yes">
		<cfargument name="cur" type="struct" required="yes">
		
		<cfscript>
			var retStruct = StructNew();
			 
			retStruct.size = arguments.default.size;
			
			if( arguments.cur.borderColor eq 'inherit' )
				retStruct.borderColor = arguments.default.borderColor;
			else
				retStruct.borderColor = arguments.cur.borderColor;
			
			return retStruct;			
		</cfscript>
		
	</cffunction>
	
		
	
	
	<cffunction name="renderPaddingStyles" access="private" output="Yes" returntype="void">
		<cfargument name="propsStruct" type="struct" required="Yes">
	
		<cfscript>
			var padding = arguments.propsStruct.defaultPadding;
			var elementID = request.element.id;
			var top = 0;
			var right = 0;
			var bottom = 0;
			var left = 0;
			var padding_xxs = '';
			var padding_xs = '';
			var padding_sm = '';
			var padding_md = '';
			var p = StructNew();
			var units = 'px,%,em,rem,vw,vh,vmin,vmax';	
			var unit = 'px';
			var i = 0;
			
			if( padding eq '' )
				return;

			p.top = val( ListGetAt(padding, 1, ' ') );
			p.right = val( ListGetAt(padding, 2, ' ') );
			p.bottom = val( ListGetAt(padding, 3, ' ') );
			p.left = val( ListGetAt(padding, 4, ' ') );
				
			for( i=1; i lte ListLen(units); i=i+1 )
			{
				unit = ListGetAt(units, i);
				if( findNoCase(unit, padding) )
					break;
			}
			
			padding_xxs = GetScaledPadding(p, .4, unit);
			padding_xs = GetScaledPadding(p, .55, unit);
			padding_sm = GetScaledPadding(p, .7, unit);
			padding_md = GetScaledPadding(p, .85, unit);
		</cfscript>
		
		<cfoutput>
			<!--- XX-Small Devices --->
			@media (max-width: 479px) { 			
				##cswTabs-#elementID# nav ul > li .cswInner { padding: #padding_xxs#; }
			}
			
			<!--- X-Small Devices --->
			@media (min-width: 480px) and (max-width: 767px)  { 
				##cswTabs-#elementID# nav ul > li .cswInner { padding: #padding_xs#; }
			}

			<!--- Small devices --->
			@media (min-width: 768px) and (max-width: 991px)  { 			
				##cswTabs-#elementID# nav ul > li .cswInner { padding: #padding_sm#; }
			}
			
			<!--- Medium devices --->
			@media (min-width: 992px) and (max-width: 1199px) { 			
				##cswTabs-#elementID# nav ul > li .cswInner { padding: #padding_md#; }
			}

			<!--- Large devices --->
			@media (min-width: 1200px) { 		
				##cswTabs-#elementID# nav ul > li .cswInner { padding: #padding#; }
			}
		</cfoutput>
		
	</cffunction>
	
	
	<cffunction name="GetScaledPadding" access="private" output="No" returntype="string">
		<cfargument name="padStruct" type="struct" required="yes">
		<cfargument name="scale" type="numeric" required="yes">
		<cfargument name="unit" type="string" required="yes">
		
		<cfscript>
			var u = arguments.unit;
			var retString = '';
			var p = arguments.padStruct;
			var newPad = StructNew();
			
			if( unit eq 'px' )
			{
				newPad.top = int(p.top * arguments.scale);
				newPad.right = int(p.right * arguments.scale);
				newPad.bottom = int(p.bottom * arguments.scale);
				newPad.left = int(p.left * arguments.scale);
			}
			else
			{
				newPad.top = NumberFormat(p.top * arguments.scale, '999.99');
				newPad.top = replaceNoCase( newPad.top, '.0', '' );
	
				newPad.right = NumberFormat(p.right * arguments.scale, '999.99');
				newPad.right = replaceNoCase( newPad.right, '.0', '' );
	
				newPad.bottom = NumberFormat(p.bottom * arguments.scale, '999.99');
				newPad.bottom = replaceNoCase( newPad.bottom, '.0', '' );
	
				newPad.left = NumberFormat(p.left * arguments.scale, '999.99');
				newPad.left = replaceNoCase( newPad.left, '.0', '' );			
			}
			
			retString = newPad.top & u & ' ' & newPad.right & u & ' ' & newPad.bottom & u & ' ' & newPad.left & u;
			
			return retString;
		</cfscript>
		
	</cffunction>
	

	<!----------------------------------------
		Get Font Classes
			- returns a string of proper css classes
	----------------------------------------->	
	<cffunction name="getFontClasses" access="public" returntype="string" output="Yes">
		<cfargument name="fontSettings" type="string" required="yes">
		
		<cfscript>
			var classes = '';
			var str = StructNew();
			var i = 0;
			var keys = 'bold,italic,underline,case';
			var item = '';
			var name = '';
			var value = '';
			
			str['bold'] = 0;			
			str['italic'] = 0;			
			str['underline'] = 0;			
			str['case'] = 'none';			
			
			for( i=1; i lte listLen(arguments.fontSettings); i=i+1 )
			{
				item = ListGetAt(arguments.fontSettings, i);
				name = ListFirst(item, ':');
				value = ListLast(item, ':');
				
				if( FindNoCase( name, keys) gt 0 )
				{
					// valid key
					str[name] = value;
				}
			}
				
			if( str.bold eq 1 )
				classes = trim(classes & ' cswTextBold');		
			if( str.italic eq 1 )
				classes = trim(classes & ' cswTextItalic');		
			if( str.underline eq 1 )
				classes = trim(classes & ' cswTextUnderline');		
			if( str.case neq 'none' )
				classes = trim(classes & ' cswText-#LCase(str.case)#');		
				
			return classes;	
		</cfscript>
	
	</cffunction>	
	
	
</cfcomponent>	
