<!--- 
	csw-tabs-vertical.cfc
	
	Version: [2018-01-02 01:28 pm]
---> 


<cfcomponent displayname="csw-tabs-vertical" output="false" hint="Vertical Tabs Functions" extends="csw-tabs-common">

	<cffunction name="loadResources" access="public" output="Yes" returntype="void">
		
		<cfscript>
			super.loadResources('cswTabsVertical');
		</cfscript>
	</cffunction>
	

	<cffunction name="loadProperties" access="public" returntype="struct" output="No">
		<cfargument name="elementInfo" type="struct" required="Yes">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">

		<cfscript>
			var widgetMetadataFormName = "CSW Tabs Vertical";
			var props = StructNew();
			var rhMetadata = arguments.ElementInfo.RenderHandlerMetaData;
			var widgetData = StructNew();
			
			// Check for Props and Layout Binding
			if( NOT hasPropsLayoutBinding(ElementInfo=arguments.elementInfo,metadataFormName=widgetMetadataFormName,rhCaller=arguments.rhCaller) )
				return props;

			widgetData = rhMetadata[widgetMetadataFormName];
			
			// Load COMMON Properties
			props = super.loadCommonProperties(widgetMetadataFormName, arguments.elementInfo, arguments.rhCaller, arguments.showDataError);
			
			
			/****************************************/
			/**** TAB BAR & CONTENT PROPERTIES  *****/
			/****************************************/
			
			// Tab Type (in common)
			// Tab Bar (in common) 
			// Content Area (in common)

			/**********************************/
			/**** DEFAULT TAB PROPERTIES  *****/
			/**********************************/
			
			// Default Tab Style (in common)
			// Deafult Padding (in common)
			// Default Border (in common)
			// Corners (in common)
			// Show Icon (in common)
			
			// Default Circle
			props.defaultCircle = getFieldSettings( widgetData=widgetData, field='Circle', defSize='3px', defBorderColor='inherit' );		
			
			/*********************************/
			/**** ACTIVE TAB PROPERTIES  *****/
			/*********************************/
			
			// Active Tab  (in common)
			// Active Border  (in common)
			// Hightlight Line  (in common)
			
			// Active Circle
			props.activeCircle = getFieldSettings( widgetData=widgetData, field='Active_Circle', defSize='inherit', defBorderColor='inherit' );		
			
			
			/********************************/
			/**** HOVER TAB PROPERTIES  *****/
			/********************************/
			
			// Hover Tab  (in common)
			// Hover Border  (in common)

			/*************************************/
			/**** GENERAL PROPERTIES TAB *********/
			/*************************************/
			
			// Overall (in common)
			// Background Image (in common)
			// Background Image Focal Point (in common)

			// Columns (STRUCT)
			props.Columns = getDeviceSizeOptions(widgetData=widgetData, field='Columns', defXSS='cswColXXS-1', defXS='cswColXS-1', defSM='cswColSM-2', defMD='cswColMD-3', defLG='cswColLG-4');
			props.colClass = getDeviceSizeClassList(props.Columns);
			
			// Accordion Scroll Offset - optional
			props.accordionScrollOffset = getStandardField( widgetData=widgetData, field='Accordion_Scroll_Offset', defValue='0' );

			
			// Overrides
			if( props.tabType contains 'cswTab-iconcircle' )
				props.showIcon eq 1;
			
			return props;
		</cfscript>
	</cffunction>
	
	
	<cffunction name="getAndRenderItems" access="public" output="Yes" returntype="void">
		<cfargument name="elementDataStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">

		<cfscript>
			var elementData = arguments.elementDataStruct;
			var props = arguments.propsStruct;
			var elementID = request.element.id;
			var tabContent = ArrayNew(1);
			var styleAttr = '';
			var outerClasses = trim(props.tabType & ' '  & props.colClass);	
			var contentStyleAttr = '';
			var accordionBreakPoint = 479;

			// Overall element styles & classes
			if( len(props.element.padding) )
					styleAttr = styleAttr & 'padding: #props.element.padding#;';
			if( len(props.element.bgColor) )
				styleAttr = styleAttr & 'color:##fff;background-color: #props.element.bgColor#;';

			if( StructKeyExists(props,'backgroundImage') AND len(props.backgroundImage) )
				styleAttr = styleAttr & "background-image: url('#props.backgroundImage#'); background-repeat: no-repeat; background-size: cover;";

			if( len(styleAttr) )
				styleAttr = 'style="#styleAttr#"';
				
			if( len(props.backgroundImageFocalPoint) )
				outerClasses = outerClasses & ' ' & props.backgroundImageFocalPoint;	
			if( len(props.defaultCorners) )
				outerClasses = outerClasses & ' ' & props.defaultCorners;	
		
			if( len(props.tabBar.align) )
				outerClasses = outerClasses & ' ' & props.tabbar.align;	

			// Content Area styles & classes
			if( len(props.contentArea.padding) )
				contentStyleAttr = contentStyleAttr & "padding: #props.contentArea.padding#;";
			if( len(props.contentArea.bgColor) )
				contentStyleAttr = contentStyleAttr & "background-color: #props.contentArea.bgColor#;";
			if( len(contentStyleAttr) )	
				contentStyleAttr = 'style="#contentStyleAttr#"';
				
			// Accordion Breakpoint
			if( props.colClass contains 'cswTabbarWidth-100-lg' )
				accordionBreakPoint = 6000;	
			else if( props.colClass contains 'cswTabbarWidth-100-md' )
				accordionBreakPoint = 1199;	
			else if( props.colClass contains 'cswTabbarWidth-100-sm' )
				accordionBreakPoint = 991;	
			else if( props.colClass contains 'cswTabbarWidth-100-xs' )
				accordionBreakPoint = 767;	
			else if( props.colClass contains 'cswTabbarWidth-100-xxs' )
				accordionBreakPoint = 479;	
			else
				accordionBreakPoint = 0;	
		</cfscript>
		
		<cfoutput>
			<section id="cswTabs-#request.element.id#" class="cswTabsVert #outerClasses# #props.showIcon#" #styleAttr#>
				<div class="tabs cswTabWrapper">
					<!--- render the tabs and return tab content --->
					<cfset tabContent = renderTabs(elementData,props)>
					<div id="cswWrapper-#request.element.id#" class="cswTabContainers" #contentStyleAttr#>
						#renderTabContainers(tabContent, props)#
					</div> 
				</div>
			</section>
			<style>
				<cfif props.tabType contains 'cswTabs-slanted'>
					#renderSlantedStyleBlock(props)#
				<cfelse>	
					#renderStyleBlock(props)#
				</cfif>
				#RenderBorderStyles(props)#
				
				<cfif accordionBreakPoint GT 0>
				@media (max-width: #accordionBreakPoint#px) {
					##cswTabs-#request.element.ID# > div > nav {
						display: none;
					}
					##cswTabs-#request.element.ID# .cswTabContent {
						display: block;
						width: 100%;
					}
					##cswTabs-#request.element.ID# .cswTabContent a.tabAccordionHeader {
						display: block;
					}
					
					##cswTabs-#request.element.ID# .cswTabContent:not(.content-current) .cswContent {
						display: none;
					}					
					##cswTabs-#request.element.ID# .cswTabContent.content-current .cswContent {
						display: block;
					}					
				}
				
				@media (min-width: #accordionBreakPoint+1#px) {
					##cswTabs-#request.element.ID# > div > nav {
						display: block;
					}
					##cswTabs-#request.element.ID# .cswTabContent a.tabAccordionHeader {
						display: none;
					}
				}
				</cfif>
			</style>
			<script>
				cswTabs_#request.element.ID#_ScrollOffset = #val(props.accordionScrollOffset)#;
			</script>
		</cfoutput>
		
	</cffunction>
	
	
	<cffunction name="renderTabs" access="public" output="Yes" returntype="array">
		<cfargument name="dataStruct" required="Yes" type="struct">
		<cfargument name="propsStruct" required="Yes" type="struct">

		<cfscript>
			var dStruct = arguments.dataStruct.propertyvalues;
			var data = StructNew();			
			var tabInfo = StructNew();			
			var props = arguments.propsStruct;			
			var numData = arraylen(dStruct);
			var elementID = request.element.id;
			var idx = 0;

			var tabActiveClass = '';
			var tabArray = arrayNew(1);
			var tabBarStyleAttr = '';
			var tabBarClasses = '';	
			
			// Tab Bar styles & classes
			if( len(props.tabBar.padding) )
				tabBarStyleAttr = tabBarStyleAttr & "padding: #props.tabBar.padding#;";
			if( len(props.tabBar.bgColor) )
				tabBarStyleAttr = tabBarStyleAttr & "background-color: #props.tabBar.bgColor#;";
				
			if( len(tabBarStyleAttr) )	
				tabBarStyleAttr = 'style="#tabBarStyleAttr#"';
			
			if( len(props.tabBar.align) )
				tabBarClasses = trim( tabBarClasses & ' #props.tabBar.align#');	
		</cfscript>

		<!--- // Tab Bar --->
		<cfoutput>
		<nav class="cswTabBar #tabBarClasses#" #tabBarStyleAttr#>
			<ul>
			</cfoutput>
	
			<cfloop index="idx" from="1" to="#numData#">
	
				<cfif structKeyExists(dStruct[idx].values,"title") and len(dStruct[idx].values.title)>
	
					<cfscript>
						data = getData(dStruct[idx].values, idx);
						tabInfo = GetTabInfo( data );
						
						tabActiveClass = '';
						if( idx EQ 1 )
							tabActiveClass = "tab-current";
					</cfscript>
						
					<!--- Render the Tab --->	
					<cfoutput>
						<li role="presentation" class="#tabActiveClass# cswTabNavItem">	
							<a href="###tabInfo.tabKey#" id="#tabInfo.tabHashID#" class="cswTabVertAnchor">					
								#RenderInnerTab(data.title, data.icon, props)#
							</a>
						</li>
					</cfoutput>
						
					<cfscript>
						arrayAppend(tabArray,tabInfo);
					</cfscript>
	
				</cfif>
	
			</cfloop>
		
			<cfoutput>	
			</ul> 
		</nav>
		</cfoutput>
		
		<cfreturn tabArray>

	</cffunction>

	
	<cffunction name="RenderInnerTab" access="private" output="yes" returntype="void">
		<cfargument name="title" type="string" required="Yes">
		<cfargument name="icon" type="string" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
		
		<cfscript>
			var props = arguments.propsStruct;
			var innerStyleAttr = '';		
			var textClasses = '';
			
			// text size
			if( len(props.defaultTab.size) )
				textClasses = props.defaultTab.size;

			// fontClasses
			props.defaultTab.fontClasses = getFontClasses( props.defaultTab.fontSettings );							
			
			// Inner Padding
			if( len(props.defaultPadding) )
				innerStyleAttr = 'style="padding: #props.defaultPadding#;"';
		</cfscript>
		
		<cfoutput>
			<div class="cswInner #props.defaultTab.fontClasses# #textClasses#" #innerStyleAttr#></cfoutput>
				<cfif len(arguments.icon) AND (props.showIcon eq 'cswTabIconAlign-left' OR props.showIcon eq 'cswTabIconAlign-top')>
					<cfoutput><i class="#arguments.icon# cswTabIcon"></i></cfoutput>
				</cfif>
				<cfoutput><span class="cswTabTitle">#arguments.title#</span></cfoutput>
				<cfif len(arguments.icon) AND props.showIcon eq 'cswTabIconAlign-right'>
					<cfoutput><i class="#arguments.icon# cswTabIcon"></i></cfoutput>
				</cfif>
			<cfoutput>	
			</div>
		</cfoutput>
	
	</cffunction>
	

	<cffunction name="renderTabContainers" access="public" output="Yes" returntype="string">
		<cfargument name="tabArray" required="Yes" type="array">
		<cfargument name="propsStruct" required="Yes" type="struct">

		<cfscript>
			var props = arguments.propsStruct;
			var data = arguments.tabArray;
			var tabContainerHTML = "";
			var contentActiveClass = "";
			var ariaHiddenState = "true";
			var idx = 0;
			var headerStyle = '';
			var headerClasses = '';
			
			var tabInfo = StructNew();
		</cfscript>

		<cfsavecontent variable="tabContainerHTML">

			<!--- // Tab Content --->
			<cfoutput>
			<div class="cswContentWrap">
			</cfoutput>

				<cfloop index="idx" from="1" to="#arraylen(data)#">

					<cfscript>
						contentActiveClass = "";
						ariaHiddenState = "true";
						if ( idx EQ 1 )
						{
							ariaHiddenState = "false";
							contentActiveClass = "content-current";
						}
					</cfscript>

					<cfoutput>
					<section id="#data[idx].tabKey#" aria-hidden="#ariaHiddenState#" role="tabpanel" aria-labelledby="#data[idx].tabKey#"
						class="#contentActiveClass# cswTabContent" data-title="#XmlFormat(data[idx].title)#">
						
						<div class="cswContentAWrapper">
							<a href="###data[idx].tabKey#" class="tabAccordionHeader cswTabVertAnchor" id="accheader_#data[idx].tabHashID#">					
								#RenderInnerTab(data[idx].title, data[idx].icon, props)#
							</a>
						</div>

						<div class="cswContent">
							<!--- // Render a container for each tab --->
							</cfoutput>

							<CFMODULE TEMPLATE="/commonspot/utilities/ct-render-named-element.cfm"
									elementtype="container"
									elementName="tabContainer_#data[idx].tabID#_#request.element.id#">

							<cfoutput>	
						</div>							
					</section>
					</cfoutput>
					
				</cfloop>

			<cfoutput>	
			</div> <!--- // END - Tab Content Column --->
			</cfoutput>

		</cfsavecontent>

		<cfreturn tabContainerHTML>

	</cffunction>
	
	
	<cffunction name="getData" access="private" output="no" returntype="struct">
		<cfargument name="values" type="struct" required="yes">
		<cfargument name="idx" type="numeric" required="yes">
		
		<cfscript>
			var data = StructNew();
			
			// Get TabID (Optional)
			// if it does NOT exists, then tabs are bound to content by their ORDER (Ugh!)
			if( StructKeyExists(arguments.values, 'tabID') )
				data.tabID = arguments.values.tabID;
			else
				data.tabID = arguments.idx;

			// Title				
			data.title = arguments.values.title;
			
			// Icon
			if( structKeyExists( arguments.values, 'icon') )
				data.icon = arguments.values.icon;
			else
				data.icon = '';
				
			return data;
		</cfscript>
	</cffunction>

	
	<cffunction name="getTabInfo" access="private" output="no" returntype="struct">
		<cfargument name="dataStruct" type="struct" required="Yes">
		
		<cfscript>
			var data = arguments.dataStruct;
			var info = StructNew();
			var elementID = request.element.id;
			
			// Build the simple tabTitleID
			info.tabTitleID = LCase( reReplace( application.ADF.data.unescapeHTMLentities(data.title), '[^a-zA-Z0-9\-\_]', '-', 'ALL' ) );
			info.tabTitleID = REReplaceNoCase(info.tabTitleID,'-{2,}','-','all');
			info.tabTitleID = ListChangeDelims(info.tabTitleID, "-", "-");

			// Build the unique TabID & Set the tab DIV IDs
			info.tabID = info.tabTitleID & "-" & elementID & "-" & data.tabID;
			info.tabHashID = 'hTabID-' & info.tabTitleID;
			info.tabKey = "hTab-" & info.tabID;
			info.tabContentKey = "hTabContent-" & info.tabID;
			
			info.title = data.title;
			info.icon = data.icon;
			
			return info;
		</cfscript>
		
	</cffunction>
	
	
	
	<cffunction name="renderStyleBlock" access="private" output="Yes" returntype="void">
		<cfargument name="propsStruct" type="struct" required="Yes">
	
		<cfscript>
			var props = arguments.propsStruct;
			var hoverFontStyles = GetFontStyles(props.hoverTab.fontSettings);
			var activeFontStyles = GetFontStyles(props.activeTab.fontSettings);
			var hoverBorderStruct = GetBorderStyles(props.defaultBorder, props.hoverBorder);
			var elementID = request.element.id;
		</cfscript>

		<!--- Default color & bgColor --->		
		<cfoutput> 
		##cswTabs-#elementID# nav > ul > li > a,
		##cswTabs-#elementID# .cswTabContent a.tabAccordionHeader {
			<cfif len(props.defaultTab.color)>
				color: #props.defaultTab.color#;
			</cfif>
			<cfif len(props.defaultTab.bgColor)>
				background-color: #props.defaultTab.bgColor#;
			</cfif>
		}
		</cfoutput>

		<!--- Hover color & bgColor --->				
		<cfif props.hoverTab.show OR props.hoverBorder.show>
			<cfoutput>
			##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:hover, 
			##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:focus,
			##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader:hover,
			##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader:focus {
				<cfif props.hoverTab.show>
					<cfif len(props.hoverTab.color)>
						color: #props.hoverTab.color#;
					</cfif>
					<cfif len(props.hoverTab.bgColor)>
						background-color: #props.hoverTab.bgColor#;
					</cfif>
					#hoverFontStyles#
				</cfif>
				<cfif props.hoverBorder.show AND len(props.hoverBorder.borderColor)>				
					border: #hoverBorderStruct.size# solid #hoverBorderStruct.borderColor#;
				</cfif>
			}
			</cfoutput>			
		</cfif>

		<!--- Active color & bgColor --->				
		<cfoutput>		
		##cswTabs-#elementID# nav > ul > li.tab-current > a,
		##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader {
			<cfif len(props.activeTab.color)>
				color: #props.activeTab.color#;
			</cfif>
			<cfif len(props.activeTab.bgColor)>
				background-color: #props.activeTab.bgColor#;
			</cfif>
			#activeFontStyles#				
		}
		</cfoutput>	
		
	</cffunction>
	
	

	<cffunction name="renderSlantedStyleBlock" access="private" output="Yes" returntype="void">
		<cfargument name="propsStruct" type="struct" required="Yes">
	
		<cfscript>
			var props = arguments.propsStruct;
			var hoverFontStyles = GetFontStyles(props.hoverTab.fontSettings);
			var defaultFontStyles = GetFontStyles(props.defaultTab.fontSettings);
			var activeFontStyles = GetFontStyles(props.activeTab.fontSettings);
			var hoverBorderStruct = GetBorderStyles(props.defaultBorder, props.hoverBorder);
			var elementID = request.element.id;
		</cfscript>

		<!--- Default color & bgColor --->		
		<cfoutput>
		##cswTabs-#elementID# nav > ul > li > a,
		##cswTabs-#elementID# .cswTabContent a.tabAccordionHeader{
			<cfif len(props.defaultTab.color)>
				color: #props.defaultTab.color#;
			</cfif>
			#defaultFontStyles#
		}
		</cfoutput>

		<!--- Hover color & bgColor --->				
		<cfif props.hoverTab.show OR props.hoverBorder.show>
			<cfoutput>
			##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:hover, 
			##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:focus,
			##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader:hover,
			##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader:focus {
				<cfif props.hoverTab.show>
					<cfif len(props.hoverTab.color)>
						color: #props.hoverTab.color#;
					</cfif>
					#hoverFontStyles#
				</cfif>
			}
			</cfoutput>			
		</cfif>

		<!--- Active color & bgColor --->				
		<cfoutput>		
		##cswTabs-#elementID# nav > ul > li.tab-current > a,
		##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader {
			<cfif len(props.activeTab.color)>
				color: #props.activeTab.color#;
			</cfif>
			#activeFontStyles#				
		}
		##cswTabs-#elementID# nav > ul > li.tab-current > a::after,
		##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader::after {
			<cfif len(props.activeTab.bgColor)>
				background-color: #props.activeTab.bgColor#;
			</cfif>
		}
		</cfoutput>		
	</cffunction>
	
	
	<cffunction name="renderBorderStyles" access="private" output="yes" returntype="void">
		<cfargument name="propsStruct" type="struct" required="Yes">
		
		<cfscript>
			var props = arguments.propsStruct;
			var activeStruct = getBorderStyles(props.defaultBorder, props.activeBorder);
			var hoverStruct = getBorderStyles(props.defaultBorder, props.hoverBorder);
			var elementID = request.element.id;
			var lineColor = '';
			var openedge = 'right';
			var closededge = 'left';

			var defaultDiameter = 0;
			var activeDiameter = 0;
			var margin = 0;
			var defaultMargin = 0;
			var activeMargin = 0;
			var pad = '';
			var padtop = 0;
			var padbot = 0;
			
			if( props.tabbar.align contains 'right' )
			{
				openedge = 'left';
				closededge = 'right';
			}
		</cfscript>
		
		<!--- STANDARD --->			
		<cfif props.tabType contains 'cswTabs-standard'>
			<cfoutput>
				<!--- DEFAULT --->
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a {
					border-#openedge#: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#; 
				}
				
				<!--- HOVER --->
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:hover, 
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:focus {
					border-bottom: none;
					border-#closededge#: #props.defaultBorder.size# solid transparent;
					border-top: none;
					border-left: none;
				}
				
				<!--- Active --->
				##cswTabs-#elementID# nav > ul > li.tab-current > a,
				##cswTabs-#elementID# nav > ul > li.tab-current > a:hover,
				##cswTabs-#elementID# nav > ul > li.tab-current > a:focus {
					border-top: #props.defaultBorder.size# solid #props.activeBorder.borderColor#; 
					border-bottom: #props.defaultBorder.size# solid #props.activeBorder.borderColor#; 
					border-#closededge#: #props.defaultBorder.size# solid #props.activeBorder.borderColor#;
				}
			</cfoutput>

		<!--- FULL --->				
		<cfelseif props.tabType contains 'cswTabs-fullborder'>
			<cfoutput>
				<!--- Default --->
				##cswTabs-#elementID# nav > ul > li > a {
					border: #props.defaultBorder.size# solid #props.defaultBorder.borderColor#;
				}					
				<!--- default tab following active tab make top border active color --->
				##cswTabs-#elementID# nav > ul > li.tab-current + li > a {
					border-top: #props.defaultBorder.size# solid #props.activeBorder.borderColor#;
				}
				
				<!--- hover --->
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:hover,
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:focus {
					border: #props.defaultBorder.size# solid #hoverStruct.borderColor#;
				}
				<!--- default tab following active tab make top border active color --->
				##cswTabs-#elementID# nav > ul > li:not(.tab-current):hover + li > a {
					border-top: #props.defaultBorder.size# solid #props.hoverBorder.borderColor#;
				}
				
				<cfif props.tabType eq 'cswTabs-fullborder'>
				##cswTabs-#elementID# nav > ul > li.tab-current > a {
					border: #props.defaultBorder.size# solid #props.activeBorder.borderColor#;
				}					
				</cfif>
				
				##cswTabs-#elementID# nav > ul > li:first-child > a {
					margin-top: 0;
				}
				##cswTabs-#elementID# nav > ul > li:not(:first-child) > a {
					margin-top: -#props.defaultBorder.size#;
				}
			</cfoutput>
			
		<!--- NONE --->				
		<cfelseif props.tabType contains 'cswTabs-noborder'>
			<cfoutput>
				##cswTabs-#elementID# nav > ul > li > a,
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:hover,
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:focus,
				##cswTabs-#elementID# nav > ul > li.tab-current > a {
					border: none;
				}
			</cfoutput>
		
		<!--- SLANTED--->          
		<cfelseif props.tabType contains 'cswTabs-slanted'>
			<cfoutput>
				##cswTabs-#elementID# nav > ul > li > a::after,
				##cswTabs-#elementID# .cswTabContent a::after {
			    	background: #props.defaultTab.bgColor#;	
				}	

				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:hover::after,
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:focus::after,
				##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader:hover::after,
				##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader:focus::after {
			    	background: #props.hoverTab.bgColor#;	
				}	
		
				##cswTabs-#elementID# nav > ul > li.tab-current > a::after,
				##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader::after {
			    	background: #props.activeTab.bgColor#;		
				}
			</cfoutput>
					
		<!--- PILLS --->				
		<cfelseif props.tabType contains 'cswTabs-pills'>
			<cfoutput>
				##cswTabs-#elementID# nav > ul > li > a,
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:hover,
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a:focus,
				##cswTabs-#elementID# nav > ul > li.tab-current > a,
				##cswTabs-#elementID# .cswTabContent a.tabAccordionHeader,
				##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader:hover,
				##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader:focus,
				##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader {
					margin-bottom: 4px;
				}
			</cfoutput>

		</cfif>
		
		
		<cfif props.tabType contains 'cswTabs-standard' OR props.tabType contains 'cswTabs-fullborder'>
			<cfoutput>
				<!--- Accordion Default --->
				##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader {
					border-#openedge#: none;	/* #props.defaultBorder.size# solid transparent */ 
				}
				<!--- Accordion Hover --->
				##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader:hover,
				##cswTabs-#elementID# .cswTabContent:not(.content-current) a.tabAccordionHeader:focus {
					border: none;
				}

				##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader,
				##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader:hover,
				##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader:focus {
					border: #props.defaultBorder.size# solid #props.activeBorder.borderColor#; 
				}
			</cfoutput>
		</cfif>
		
	
		<!---// HIGHLIGHT STYLES //---------->		
		<cfif props.tabType contains 'cswTabs-slanted'>
			<cfreturn>
		</cfif>
		
		<cfif props.tabType contains 'cswTab-lineabove'>
			<cfoutput>
			##cswTabs-#elementID# nav > ul > li.tab-current > a,
			##cswTabs-#elementID# nav > ul > li.tab-current > a:hover,
			##cswTabs-#elementID# nav > ul > li.tab-current > a:focus,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader:hover,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader:focus {
				box-shadow: inset 0px #props.highlightLine.size# 0px 0px #props.highlightLine.borderColor#;
			}
			</cfoutput>
		<cfelseif props.tabType contains 'cswTab-linebelow'>
			<cfoutput>
			##cswTabs-#elementID# nav > ul > li.tab-current > a,
			##cswTabs-#elementID# nav > ul > li.tab-current > a:hover,
			##cswTabs-#elementID# nav > ul > li.tab-current > a:focus,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader:hover,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader:focus {
				box-shadow: inset 0px -#props.highlightLine.size# 0px 0px #props.highlightLine.borderColor#;
			}
			</cfoutput>
		<cfelseif props.tabType contains 'cswTab-lineleft'>
			<cfoutput>
			##cswTabs-#elementID# nav > ul > li.tab-current > a,
			##cswTabs-#elementID# nav > ul > li.tab-current > a:hover,
			##cswTabs-#elementID# nav > ul > li.tab-current > a:focus,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader:hover,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader:focus {
				box-shadow: #props.highlightLine.size# 0px 0px 0px #props.highlightLine.borderColor# inset;	
			}
			</cfoutput>
		<cfelseif props.tabType contains 'cswTab-lineright'>
			<cfoutput>
			##cswTabs-#elementID# nav > ul > li.tab-current > a,
			##cswTabs-#elementID# nav > ul > li.tab-current > a:hover,
			##cswTabs-#elementID# nav > ul > li.tab-current > a:focus,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader:hover,
			##cswTabs-#elementID# .cswTabContent.content-current a.tabAccordionHeader:focus {
				box-shadow: -#props.highlightLine.size# 0px 0px 0px #props.highlightLine.borderColor# inset;
			}
			</cfoutput>
		<cfelseif props.tabType contains 'cswTab-iconcircle'>
			<cfscript>
				// make sure border is at least 1px
				if( len(props.defaultCircle.style) eq 0 OR val(props.defaultCircle.style) lt 1 )
					props.defaultCircle.style = '1px';
					
				if( len(props.activeCircle.style) eq 0 OR props.activeCircle.style eq 'inherit' )
					props.activeCircle.style = props.defaultCircle.style;
				else if( val(props.activeCircle.style) lt 1 )
					props.activeCircle.style = '1px';
				else
					props.activeCircle.style = props.activeCircle.style;

					
				// set diameter and needed margins	
				defaultDiameter = val(props.defaultCircle.size);
				if( defaultDiameter lt 40 )
					defaultDiameter = 40;
				defaultMargin = (defaultDiameter / 2) + 10;
				
				// set diameter and needed margins	
				if( len(props.activeCircle.size) eq 0 OR props.activeCircle.size eq 'inherit' )
					activeDiameter = defaultDiameter;
				else
					activeDiameter = val(props.activeCircle.size);
				
				if( activeDiameter lt 40 )
					activeDiameter = 40;
				activeMargin = (activeDiameter / 2) + 10;

				diameter = Max(defaultDiameter, activeDiameter);
				margin = Max(defaultMargin, activeMargin);

				// make sure padding around the tab is tall enough
				pad = props.defaultPadding;
				padtop = val(ListFirst(pad, ' '));
				padbot = val(ListGetAt(pad, 3, ' '));
			</cfscript>
			
			<cfoutput>
				##cswTabs-#elementID# nav > ul > li > a,
				##cswTabs-#elementID# .cswTabContent a {
					<cfif props.defaultCircle.align eq 'right'>	
						margin-right: #margin#px;
						text-align: right;
					<cfelse>
						margin-left: #margin#px;
						text-align: left;
					</cfif>
				}

				##cswTabs-#elementID# nav > ul > li > a i,
				##cswTabs-#elementID# .cswTabContent a i {
					<cfif props.defaultCircle.align eq 'right'>	
						right: 0;
						transform: translate(50%,-50%);
					<cfelse>
						left: 0;
						transform: translate(-50%,-50%);
					</cfif>
				}

				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a i,
				##cswTabs-#elementID# .cswTabContent:not(.content-current) a i {
					<cfif len(props.defaultCircle.borderColor) eq 0 OR props.defaultCircle.borderColor eq 'inherit'>
						border-color: #props.defaultBorder.borderColor#;
					<cfelse>
						border-color: #props.defaultCircle.borderColor#;
					</cfif>
					border-width: #val(props.defaultCircle.style)#px;
					width: #defaultDiameter#px; 
					height: #defaultDiameter#px; 
					line-height: #defaultDiameter - (val(props.defaultCircle.style)*2)#px; 
					background-color: #props.defaultTab.bgColor#;
				}
				##cswTabs-#elementID# nav > ul > li.tab-current > a i,
				##cswTabs-#elementID# .cswTabContent.content-current a i {
					<cfif len(props.activeCircle.borderColor) eq 0 OR props.activeCircle.borderColor eq 'inherit'>
						<cfif len(props.activeBorder.borderColor) eq 0 OR props.activeBorder.borderColor eq 'inherit'>
							border-color: #props.defaultBorder.borderColor#;
						<cfelse>	
							border-color: #props.activeBorder.borderColor#;
						</cfif>
					<cfelse>
						border-color: #props.activeCircle.borderColor#;
					</cfif>
					border-width: #val(props.activeCircle.style)#px;
					width: #activeDiameter#px; 
					height: #activeDiameter#px; 
					line-height: #activeDiameter - (val(props.activeCircle.style)*2)#px; 
					background-color: #props.activeTab.bgColor#;
				}
				##cswTabs-#elementID# nav > ul > li:not(.tab-current) > a .cswTabTitle,
				##cswTabs-#elementID# .cswTabContent:not(.content-current) a .cswTabTitle {
					<cfif props.defaultCircle.align eq 'right'>	
						margin-right: #defaultMargin#px;
					<cfelse>
						margin-left: #defaultMargin#px;
					</cfif>
				}
				##cswTabs-#elementID# nav > ul > li.tab-current > a .cswTabTitle,
				##cswTabs-#elementID# .cswTabContent.content-current a .cswTabTitle {
					<cfif props.defaultCircle.align eq 'right'>	
						margin-right: #activeMargin#px;
					<cfelse>
						margin-left: #activeMargin#px;
					</cfif>
				}
				<cfif padTop lt val(diameter/2)>
					##cswTabs-#elementID# nav > ul > li > a .cswInner,
					##cswTabs-#elementID# .cswTabContent a .cswInner {
						padding-top: #val(diameter/2)#px !important;
					}
				</cfif>
				<cfif padBot lt val(diameter/2)>
					##cswTabs-#elementID# nav > ul > li > a .cswInner,
					##cswTabs-#elementID# .cswTabContent a .cswInner {
						padding-bottom: #val(diameter/2)#px !important;
					}
				</cfif>				
			</cfoutput>
		</cfif>
				               
		<cfif props.tabType contains 'cswTab-triangle'>
			<cfoutput>
			##cswTabs-#elementID# nav ul > li.tab-current > a::after {
				<cfif props.tabbar.align contains 'left'>
			   	border-left-color: #props.activeTab.bgColor#;
				<cfelse>
					border-right-color: #props.activeTab.bgColor#;
				</cfif> 
			}
			</cfoutput>		
		</cfif>

	</cffunction>

</cfcomponent>